'\"
'\" Soar man page 
'\"
.so man.macros
.HS print soar
.BS
'\" Note:  do not modify the .SH NAME line immediately below!
.SH NAME
print \- Print a Soar item from production memory or working memory
.SH SYNOPSIS
\fBprint\fR -stack [ \fB-state\fR | \fB-operator\fR ]*
.br
\fBprint\fR [\fB-depth\fR \fIn\fR] [\fB-internal\fR] {\fI identifier | integer | pattern\fR}
.br
\fBprint\fR [\fB-internal\fR] [\fB-name|-full\fR] {\fI prodname | -all | -chunks | -defaults | -justifications | -user\fR}
.BE

.SH DESCRIPTION
.PP
The \fBprint\fR command is used to print items from production 
memory or working memory.  It can take several kinds of arguments.  When 
printing items from working memory, the Soar objects are printed unless the
\fB-internal\fR flag is used, in which case the wmes themselves are printed.
 The \fIitem\fRs to be printed may be one of the following:
.TP
\fIproduction-name\fR
print the production named \fIproduction-name\fR
.TP
\fI-all\fR
print the names of all productions currently loaded
.TP
\fI-chunks\fR
print the names of all chunks currently loaded
.TP
\fI-defaults\fR
print the names of all default productions currently loaded
.TP
\fI-justifications\fR
print the names of all justifications currently loaded
.TP
\fI-user\fR
print the names of all user productions currently loaded
.TP
\fIidentifier\fR
print the object \fIidentifier\fR.  \fIidentifier\fR must be a
valid Soar symbol.
.TP
\fIinteger\fR
print the object in working memory with the timetag \fIinteger\fR
.TP
\fIpattern\fR
print the object whose working memory elements matching the given 
\fIpattern\fR.
The pattern is interpreted by the Soar kernel so it must be surrounded
by curly braces or double quotes to avoid being parsed by Tcl.  A
\fIpattern\fR has the following form:
.DS
       \fB(\fIidentifier \fB^\fIattribute value\fR\fR [\fB\fB+\fR]\fB)\fR
.DE
.PP
The \fIpattern\fR is surrounded by parentheses.  The \fIidentifier\fR,
\fIattribute\fR, and \fIvalue\fR must be valid Soar symbols or the
wildcard symbol \fB*\fR which matches all occurences.  The optional
\fB+\fR symbol restricts pattern matches to acceptable preferences.

.SH OPTIONS
.PP
All options apply only to arguments which follow them on the command line.
.TP
\fB-depth\fR \fIn\fR 
This option overrides the default
printing depth (see the \fBdefault-print-depth\fR command for more
detail).  Note that a \fB-depth 0\fR argument is meaningful only with
accompanying integer and pattern arguments.  It causes only the 
matching working memory elements to be printed, instead of all 
elements whose identifier is an identifier in one of the matching 
working memory elements.
.TP
\fB-internal\fR 
Specifies that items should be printed 
in their internal form.  For productions, this means leaving 
conditions in their reordered (rete net) form.  For working memory, 
this means printing the individual elements with their 
timetags, rather than the objects.
.TP
\fB-full\fR 
When printing productions, print the whole production.  This is the
default when printing a named production.
.TP
\fB-name\fR 
When printing productions, print only the name and not the whole
production.  This is the default when printing any catorgory of
productions, as opposed to a named production.
.TP
\fB-stack\fR
Specifies that the Soar goal stack should be printed.  By default
this includes both states and operators.  The stack listing can
be restricted by adding the \fB-states\fR and \fB-operator\fR
restrictions.  Giving both options is equivalent to the default.
The predefined alias 'ps' is the same as \fBprint -stack\fR.

.SH EXAMPLES
.TP
\fBprint -internal {(s1 ^* v2)}\fR
Print the working memory elements (and their timetags) which have
the identifier s1 as object and v2 as value.
.TP
\fBprint -stack\fR
Print the Soar stack which includes states and operators.
.TP
\fBprint -internal -full prodname\fR
Print the named production in its RETE form.
.TP
\fBprint -user\fR
Print the names of all user productions currently loaded.

.SH WARNINGS

The order of options and arguments to the \fBprint\fR command
do matter:  options apply only to arguments which follow them.

.TP
\fB print -chunks -full\fR
prints only the names of chunks, while
.TP
\fB print -full -chunks \fR
prints the entire chunk for all chunks.

.SH SEE ALSO
\fBdefault-wme-depth\fR, \fBpredefined-aliases\fR









