'\"
'\" Soar man page 
'\"
.so man.macros
.HS stats soar
.BS
'\" Note:  do not modify the .SH NAME line immediately below!
.SH NAME
stats \- Print statistical data about Soar
.SH SYNOPSIS
\fBstats \fR[\fI-system \fR[\fIstat\fR] | \fI-memory \fR[\fIstat\fR] |\fI-rete \fR[\fIstat\fR]]
.BE

.SH DESCRIPTION
.PP
This command prints Soar internal statistics.  The \fImodule\fR
indicates the component of interest.  If specified, \fImodule\fR 
must be one of \fB-system\fR, \fB-memory\fR, or \fB-rete\fR.  If no 
specific \fIstat\fR is given for the specified \fImodule\fR, then
all statistics are listed for that \fImodule\fR.  If no \fImodule\fR
is given, then the \fB-system\fR statistics are listed.
.PP
The \fB-system\fR module contains the following \fIstat\fRs:
.DS
-default-production-count
-user-production-count
-chunk-count
-justification-count
-all-productions-count
-dc-count
-ec-count
-ecs/dc
-firings-count
-firings/ec
-wme-change-count
-wme-addition-count
-wme-removal-count
-wme-count
-wme-avg-count
-wme-max-count
.DE
.PP
If Soar has been compiled with the NO_TIMING_STUFF \fBNOT\fR set,
then the following statistics are also available in the 
\fB-system\fR module:
.DS
-total-time
-ms/dc
-ms/ec
-ms/firing
-ms/wme-change
.DE
.PP
If Soar has been compiled with the DETAILED_TIMING_STATS \fBSET\fR,
then the following statistics are also available in the 
\fB-system\fR module:
.DS
-match-time
-ownership-time
-chunking-time
.DE
.PP
The \fB-memory\fR module contains the following statistics:
.DS
-total
-overhead
-strings
-hash-table
-pool \fIpool-statistic\fR
-misc
.DE
.PP
The \fB-pool\fR statistic is additionally qualified by a 
\fIpool-statistic\fR option.  If this option is not given,
then all \fB-memory\fR pool statistics are printed.  If
given, \fIpool-statistic\fR must have one of the following 
forms:
.TP
\fB-total\fR
Print just the overall totals for the pools
.TP
\fIpool-name\fR [\fIaspect\fR]
Print statistics about the specific pool \fIpool-name\fR.  The
permissable \fIpool-name\fRs are:
.DS
     chunk condition
     io wme
     output link
     preference
     wme
     slot
     instantiation
     ms change
     right mem
     negative token
     token
     node varnames
     rete node
     rete test
     alpha mem
     saved test
     not
     action
     production
     condition
     complex test
     float constant
     int constant
     sym constant
     identifier
     variable
     dl cons
     cons cell
.DE
.PP
If no \fIaspect\fR is given, then print all statistics about the
given pool.  If given, \fIaspect\fR must have one of the
following forms:
.DS
     -used
     -free
     -item-size
     -total-bytes
.DE
The first two \fIaspect\fRs are available only if Soar has been
compiled with MEMORY_POOL_STATS \fBSET\fR.  
.PP
The \fB-rete\fR module contains statistics for the following node
types:
.DS
unhashed memory
memory
unhashed mem-pos
mem-pos
unhashed negative
negative
unhashed positive
positive
dummy top
dummy matches
conj. neg.
conj. neg. partner
production
total
.DE
The \fBtotal\fR statistic gives a total count over all node types.
.PP
A \fB-rete\fR statistic is obtained by specifying one of the above
nodes followed by a \fIqualifier\fR.  The \fIqualifier\fR is one 
of \fB-actual\fR, \fB-if-no-merging\fR, and \fB-if-no-sharing\fR.
The \fB-if-no-sharing\fR option is only available if Soar has been
compiled with the SHARING_FACTORS option \fBSET\fR.

.SH EXAMPLES
This prints all statistics in the \fB-system\fR module:
.DS
\fBstats -system\fR
.DE
.PP
This prints the number of io wme's:
.DS
\fBstats -memory -pool {io wme} -used\fR
.DE
Note that this particular pool name, "io wme", contains an
embedded space character.  Hence, the name must be surrounded
by curly braces or double quotes to override the normal Tcl
parsing of words separated by whitespace.  This is not necessary
for pool names that do not include spaces:
.DS
\fBstats -memory -pool wme -used\fR
.DE
.PP
This prints the actual number of unhashed negative nodes in the 
Rete memory:
.DS
\fBstats -rete {unhashed negative} -actual\fR
.DE

.SH A NOTE ON TIMERS
.PP
The current implementation of Soar uses a number of timers to 
provide time-based statistics for use in the \fBstats\fR command
calculations.  These timers are:
.DS
total CPU time
total kernel time
phase kernel time (per phase)
phase callbacks time (per phase)
input function time
output function time
.DE
.PP

Total CPU time is calculated from the time a decision cycle (or number
of decision cycles) is initiated until stopped.  Kernel time is the
time spent in core Soar functions.  In this case, kernel time is
defined as the all functions other than the execution of callbacks and
the input and output functions.  The total kernel timer is only
stopped for these functions.

The phase timers (for the kernel and callbacks) track the execution
time for individual phases of the decision cycle (i.e., input phase,
preference phase, working memory phase, output phase, and decision
phase).  Because there is overhead associated with turning these
timers on and off, the actual kernel time will always be greater than
the derived kernel time (i.e., the sum of all the phase kernel
timers).  Similarly, the total CPU time will always be greater than
the derived total (the sum of the other timers) because the overhead
of turning these timers on and off is included in the total CPU time.
In general, the times reported by the single timers should always be
greater than than the corresponding derived time.  Additionally, as
execution time increases, the difference between these two values will
also increase.

For those concerned about the performance cost of the timers, all the
run time timing calculations can be compiled out of the code by
defining NO_TIMING_STUFF (in soarkernel.h) before compilation.

.PP
.SH FILES
soarkernel.h




