/*
 * =======================================================================
 *  File:  soarScheduler.c
 *
 * Uses a round-robin to run all soar agents.
 *
 * =======================================================================
 *
 *
 * Copyright (c) 1995-1999 Carnegie Mellon University,
 *                         University of Michigan,
 *                         University of Southern California/Information
 *                         Sciences Institute.
 *
 * The Soar consortium proclaims this software is in the public domain, and
 * is made available AS IS.  Carnegie Mellon University, The University of 
 * Michigan, and The University of Southern California/Information Sciences 
 * Institute make no warranties about the software or its performance,
 * implied or otherwise.
 * =======================================================================
 */


#include "soar.h"
#include "soarCommands.h"
#include "soarCommandUtils.h"
#include "soarScheduler.h"

long scheduler_cycle_count = 0;

Soar_ScheduledInterp Soar_BeforeSchedule = {NULL, NULL};
Soar_ScheduledInterp Soar_AfterSchedule = {NULL, NULL};

int
schedule_agents (Tcl_Interp * interp, int cycles) 
{
  bool agent_to_run = TRUE;
  cons  * c; 
  agent * the_agent; 
  agent * prev_agent;
  int cycle_count = 0;

  prev_agent = soar_agent;

  for(c = all_soar_agents; c != NIL; c = c->rest) {
    the_agent = (agent *) c->first;
    the_agent->stop_soar = FALSE;
  }

  while (TRUE) {

    if (cycle_count == cycles) break;
    cycle_count++;

    agent_to_run = FALSE;
    for(c = all_soar_agents; c != NIL; c = c->rest) {
      soar_agent = (agent *) c->first;
      if (!(soar_agent->stop_soar)) {
	agent_to_run = TRUE;
      }
    }

    if (!agent_to_run) break;

    if (Soar_BeforeSchedule.interp != NULL) {
      int code;

      code = Tcl_GlobalEval(Soar_BeforeSchedule.interp, 
			    Soar_BeforeSchedule.script);

      if (code != TCL_OK)
	{
	  interp->result = Soar_BeforeSchedule.interp->result;
	  return TCL_ERROR;
	}
    }

    for(c = all_soar_agents; c != NIL; c = c->rest) {
      soar_agent = (agent *) c->first;

      Soar_SelectGlobalInterpByInterp(soar_agent->interpreter);

     if (!current_agent(stop_soar)) {

       soar_invoke_callbacks(soar_agent, 
			     BEFORE_SCHEDULE_CYCLE_CALLBACK,
			     (soar_call_data) TRUE);

       execute_go_selection(soar_agent,
			    soar_agent->go_number,
			    soar_agent->go_type,
			    soar_agent->go_slot_attr,
			    soar_agent->go_slot_level);

       soar_invoke_callbacks(soar_agent, 
			     AFTER_SCHEDULE_CYCLE_CALLBACK,
			     (soar_call_data) TRUE);
     } /* if */
    } /* for */

    if (Soar_AfterSchedule.interp != NULL) {
      int code;

      code = Tcl_GlobalEval(Soar_AfterSchedule.interp, 
			    Soar_AfterSchedule.script);

      if (code != TCL_OK)
	{
	  interp->result = Soar_AfterSchedule.interp->result;
	  return TCL_ERROR;
	}
    }


  } /* while */

  soar_agent = prev_agent;

  Soar_SelectGlobalInterpByInterp(soar_agent->interpreter);

  return TCL_OK;
}
