##########################################################################
# File         : tsiControlPanel.tcl
# Author       : Gordon D. Baxter
# Date	       : 01/07/96
# Description  : 
#
#
# This version of the control panel was primarily written by Randy Jones.
# 
##########################################################################

###
###	I.	Make the TSI Control Panel
###

proc makeTSIControlPanel {{hide 0} {x -20} {y 1}} {

  if $hide {
    wm withdraw .
  } else {
    wm deiconify .
  }
  . configure -relief ridge -borderwidth 5
  wm title . {Agent Control Panel}
  if {$x >= 0} { set x +$x }
  if {$y >= 0} { set y +$y }
  wm geometry . $x$y

  # create the menu bar
  frame .menu -relief raised -bd 2
  pack .menu -side top -fill x

  menubutton .menu.file -text {File} -menu .menu.file.m

  menu .menu.file.m 
  .menu.file.m add command -label Exit -command quitSoar

  pack .menu.file -side left -padx 5

  # create a frame for the general buttons

  frame .run -relief sunken -borderwidth 2
  button .run.step -text Step -command tsiStep-soar
  button .run.run -text Run -command tsiRun-soar
  button .run.stop -text Stop -command fake-stop-soar
  button .run.quit -text Quit -command quitSoar

  # pack the frame, and its components
  pack .run.step .run.run .run.stop .run.quit -side top -anchor n -fill both
  pack .run -side right -fill y

  frame .agents
  frame .agents.create -relief sunken -borderwidth 2
  label .agents.create.name -text {Name:}
  entry .agents.create.e -width 20 -relief sunken -textvariable newAgentName
  button .agents.create.b -text {Create New Agent} -relief raised \
         -command {createNewAgent $newAgentName; set newAgentName {}}

  pack .agents.create.name .agents.create.e .agents.create.b -side left
  pack .agents.create -side top -fill x

  ## no click to focus
  bind . <Enter> { focus .agents.create.e }
  bind . <Return> { .agents.create.b invoke }

  # The rest of the window will be filled with agent frames as they
  # are registered with the control panel.
  # There will be a permanent frame at the top for commands to be
  # issued to all agents at the same time

  frame .agents.agents -relief ridge -borderwidth 5
  pack .agents.agents -side top -fill x

  label .agents.agents.label -text {Agent Menus:}
  menubutton .agents.agents.all -text {All Agents} -relief raised \
             -menu .agents.agents.all.m

  menu .agents.agents.all.m
  .agents.agents.all.m add command -label {Open All Agent Windows} \
       -command {foreach a [tsiListAgents] { \
                    set showAgent($a) 1 ;\
                    tsiSendAgent $a {wm deiconify .tsw} \
                 }}
  .agents.agents.all.m add command -label {Close All Agent Windows} \
       -command {foreach a [tsiListAgents] { \
                    set showAgent($a) 0 ;\
                    tsiSendAgent $a {wm withdraw .tsw} \
                 }}
  .agents.agents.all.m add command -label {Watch 0} \
       -command {foreach a [tsiListAgents] { \
                    tsiSendAgent $a {tsiDisplayAndSendCommand {watch 0}} \
                 }}
  .agents.agents.all.m add separator
  .agents.agents.all.m add command -label {Clear Working Memory} \
         -command {sendAllAgents init-soar}
  .agents.agents.all.m add command -label {Excise All} \
         -command {sendAllAgents {excise -all}}
  .agents.agents.all.m add command -label {Excise Chunks} \
         -command {sendAllAgents {excise -chunks}}

  pack .agents.agents.label .agents.agents.all -side left -fill y
  pack .agents -side top -padx 10 -pady 10
}


###
###	II.	Helper functions
###

### For Soar 7.1, assume that all agents are in slave interpreters
### to the controller.  So if people try to get creative with their
### interpreter hierarchies, this will fail.
proc tsiListAgents {} {
   if [catch {set x [interp slaves]} m] {
      error "Cannot compute list of agents: $m"
   }
   return $x
}

### need to worry about having whitespace in the agent name...that
### can break things RMJ
proc createNewAgent {name} {
   global auto_path tsi_library soar_library soar_doc_dir tsiConfig
   if [catch "interp create $name"] {
      error "Cannot create agent named '$name'"
   }
   load {} Tk $name
   $name eval [list set auto_path $auto_path]
   $name eval {package require Soar}
   $name eval [list set soar_library $soar_library]
   $name eval [list set tsi_library $tsi_library]
   if [info exists soar_doc_dir] {
      $name eval [list set soar_doc_dir $soar_doc_dir]
   }
   ### Configure the agent for the TSI
   set id [array startsearch tsiConfig]
   while {[array anymore tsiConfig $id]} {
      set elm [array nextelement tsiConfig $id]
      $name eval [list set tsiConfig($elm) $tsiConfig($elm)]
   }
   $name eval [list set tcl_interactive 1]  ;# want command completion...

   array donesearch tsiConfig $id
   $name eval [list set interp_name $name]
   $name alias registerWithController registerAgent $name
   $name alias tsiListAgents tsiListAgents
   $name eval tsiInitAgent
	incr tsiConfig(window_count)

}

proc tsiSendAgent {agent command} {
   ### Need to use {} for catch, or $command gets concat'ed into a list
   if [catch {$agent eval $command} msg] {
      error "Error sending command '$command' to agent $agent!"
   }
}

### New version using slave interpreters in Soar 7.1
proc sendAllAgents {msg} {
   foreach i [tsiListAgents] {
      if [catch {$i eval $msg} m] {
         error "Error sending message $msg to agent $i: $m"
      }
   }
}

proc registerAgent {name} {
  global showAgent
  menubutton .agents.agents.agent$name -text $name -relief raised \
              -menu .agents.agents.agent$name.m

  menu .agents.agents.agent$name.m
  .agents.agents.agent$name.m add checkbutton -label {Show Agent Window} \
       -variable showAgent($name) \
       -command "agentToggleCommand $name \$showAgent($name) \
                   {wm deiconify .tsw} {wm withdraw .tsw}"
  set showAgent($name) 1
  .agents.agents.agent$name.m add command -label {Raise Agent Window} \
       -command "tsiSendAgent $name {raise .tsw}"
  .agents.agents.agent$name.m add command -label {watch 0} \
      -command "tsiSendAgent $name {tsiDisplayAndSendCommand {watch 0}}"
  .agents.agents.agent$name.m add separator
  .agents.agents.agent$name.m add command -label {Clear Working Memory} \
         -command "tsiSendAgent $name {tsiDisplayAndSendCommand init-soar}"
  .agents.agents.agent$name.m add command -label {Excise All} \
         -command "tsiSendAgent $name {tsiDisplayAndSendCommand {excise -all}}"
  .agents.agents.agent$name.m add command -label {Excise Chunks} \
         -command "tsiSendAgent $name {tsiDisplayAndSendCommand {excise -chunks}}"
  .agents.agents.agent$name.m add separator
  .agents.agents.agent$name.m add command -label {Destroy Agent} \
      -command "destroyAgent $name"

  pack .agents.agents.agent$name -side left -fill y
}

proc agentToggleCommand {name t on off} {
   if $t {
      tsiSendAgent $name $on
   } else {
      tsiSendAgent $name $off
   }
}

proc destroyAgent {name} {
   if {[lsearch [tsiListAgents] $name] >= 0} {
      if [catch "destroy-interpreter $name"] {
         if [catch "interp delete $name"] {
            error "Cannot delete agent '$name'!"
         }
      }
   }
   if [winfo exists .agents.agents.agent$name] {
      destroy .agents.agents.agent$name
   }
}

### Need a fake-stop-soar, because stop-soar is not defined in wish
### interpreters
proc fake-stop-soar {} {
   set a [tsiListAgents]
   if {$a == {}} {
      tk_dialog .error {Error} \
                       {Can't stop!  No Soar interpreters are running!}\
                error 0 Dismiss
      return
   }
   tsiSendAgent [lindex $a 0] stop-soar
}

proc tsiRun-soar {} {
   set a [tsiListAgents]
   if {$a == {}} {
      tk_dialog .error {Error} \
                       {Can't run!  No Soar interpreters exist!}\
                error 0 Dismiss
      return
   }
   tsiSendAgent [lindex $a 0] run
}
proc tsiStep-soar {} {
   set a [tsiListAgents]
   if {$a == {}} {
      tk_dialog .error {Error} \
                       {Can't step!  No Soar interpreters exist!}\
                error 0 Dismiss
      return
   }
   tsiSendAgent [lindex $a 0] "run 1"
}
